@@ -10,6 +10,9 @@ from datetime import datetime, timedelta
 from homeconnect_webthing.auth import Auth
 
 
+def is_success(status_code: int) -> bool:
+    return status_code >= 200 and status_code <= 299
+
 
 
 class EventListener(ABC):
@@ -48,14 +51,11 @@ class Device(EventListener):
     def is_dishwasher(self) -> bool:
         return False
 
-    def __is_success(self, status_code: int) -> bool:
-        return status_code >= 200 and status_code <= 299
-
     def _perform_get(self, path:str) -> Dict[str, Any]:
         uri = self._uri + path
         #logging.info("query GET " + uri)
-        response = requests.get(uri, headers={"Authorization": "Bearer " + self._auth.access_token})
-        if self.__is_success(response.status_code):
+        response = requests.get(uri, headers={"Authorization": "Bearer " + self._auth.access_token}, timeout=5000)
+        if is_success(response.status_code):
             return response.json()
         else:
             logging.warning("error occurred by calling GET " + uri)
@@ -64,9 +64,8 @@ class Device(EventListener):
 
     def _perform_put(self, path:str, data: str, max_trials: int = 3, current_trial: int = 1):
         uri = self._uri + path
-        #logging.info("query PUT " + uri + " (" + str(current_trial) + " trial)")
-        response = requests.put(uri, data=data, headers={"Content-Type": "application/json", "Authorization": "Bearer " + self._auth.access_token})
-        if not self.__is_success(response.status_code):
+        response = requests.put(uri, data=data, headers={"Content-Type": "application/json", "Authorization": "Bearer " + self._auth.access_token}, timeout=5000)
+        if not is_success(response.status_code):
             logging.warning("error occurred by calling PUT " + uri + " " + data)
             logging.warning("got " + str(response.status_code) + " " + str(response.text))
             if current_trial <= max_trials:
@@ -105,13 +104,16 @@ class Dishwasher(Device):
         self.__program_selected = ""
         self.remote_start_allowed = False
         self.__program_start_in_relative_sec = 0
-        self.__program_remaining_time = ""
         self.__program_progress = 0
         self.__program_active = ""
         self.__program_remote_control_active = ""
+        self.program_remaining_time_sec = ""
         self.program_extra_try = ""
         self.program_hygiene_plus = ""
         self.program_vario_speed_plus = ""
+        self.program_energy_forecast = ""
+        self.program_water_forecast = ""
+
         self._value_changed_listeners = set()
         super().__init__(uri, auth, name, device_type, haid, brand, vib, enumber)
         self.__refresh()
@@ -133,8 +135,7 @@ class Dishwasher(Device):
         self.__refresh()
 
     def on_keep_alive_event(self, event):
-        #logging.debug("keep alive event")
-        pass
+        logging.debug("keep alive event")
 
     def on_notify_event(self, event):
         #logging.debug("notify event: " + str(event.data))
@@ -178,7 +179,7 @@ class Dishwasher(Device):
             elif record['key'] == 'BSH.Common.Option.StartInRelative':
                 self.__program_start_in_relative_sec = record['value']
             elif record['key'] == 'BSH.Common.Option.RemainingProgramTime':
-                self.__program_remaining_time = record['value']
+                self.program_remaining_time_sec = record['value']
             elif record['key'] == 'BSH.Common.Option.ProgramProgress':
                 self.__program_progress = record['value']
             elif record['key'] == 'BSH.Common.Status.RemoteControlActive':
@@ -189,17 +190,30 @@ class Dishwasher(Device):
                 self.program_hygiene_plus = record['value']
             elif record['key'] == 'Dishcare.Dishwasher.Option.VarioSpeedPlus':
                 self.program_vario_speed_plus = record['value']
+            elif record['key'] == 'BSH.Common.Option.EnergyForecast':
+                self.program_energy_forecast = record['value']
+            elif record['key'] == 'BSH.Common.Option.WaterForecast':
+                self.program_water_forecast = record['value']
             else:
                 logging.info("unknown changed " + str(record))
 
     def __refresh(self, notify: bool = True):
         try:
             logging.info("fetch settings, status and selection")
-            self.__on_value_changes(self._perform_get('/settings')['data']['settings'], "fetched")
-            self.__on_value_changes(self._perform_get('/status')['data']['status'], "fetched")
+            settings = self._perform_get('/settings')['data']['settings']
+            logging.info("settings, status and selection fetched")
+            self.__on_value_changes(settings, "fetched")
+
+            status = self._perform_get('/status')['data']['status']
+            self.__on_value_changes(status, "fetched")
+
             record = self._perform_get('/programs/selected')['data']
             self.__program_selected = record['key']
             self.__on_value_changes(record['options'], "fetched")
+
+            record = self._perform_get('/programs/active')['data']
+            self.__on_value_changes(record['options'], "fetched")
+
             if notify:
                 self.__notify_listeners()
         except Exception as e:
@@ -296,19 +310,23 @@ class HomeConnect:
     def __listening_for_events(self):
         sleep(3)
         uri = HomeConnect.API_URI + "/homeappliances/events"
-        num_reconnects = 0
+
         while True:
+            start_time = datetime.now()
             try:
-                self.__consume_sse_events(uri, read_timeout_sec=4 * 60, max_lifetime_sec=45 * 60)
-                num_reconnects = 0
+                self.__consume_sse_events(uri, read_timeout_sec=15 * 60, max_lifetime_sec=90 * 60)
             except Exception as e:
                 logging.warning("Event stream (" + uri + ") error: ", e)
-                wait_time_sec = {0: 3, 1:5, 2: 30, 3: 2*60, 4: 5*60}.get(num_reconnects, 20*60)
-                num_reconnects += 1
-                logging.info("try " + str(num_reconnects) + ". reconnect in " + str(wait_time_sec) + " sec")
+                elapsed_min = (datetime.now() - start_time).total_seconds() / 60
+                wait_time_sec = 5 * 60
+                if elapsed_min > 30:
+                    wait_time_sec = 5
+                logging.info("try reconnect in " + str(wait_time_sec) + " sec")
                 sleep(wait_time_sec)
+                logging.info("reconnecting")
 
     def __consume_sse_events(self, uri: str, read_timeout_sec: int, max_lifetime_sec:int):
+        connect_time = datetime.now()
         client = None
         try:
             logging.info("opening event stream connection " + uri + "(read timeout " + str(read_timeout_sec) + " sec, lifetimeout " + str(max_lifetime_sec) + " sec)")
@@ -316,13 +334,12 @@ class HomeConnect:
                                     stream=True,
                                     timeout=read_timeout_sec,
                                     headers={'Accept': 'text/event-stream', "Authorization": "Bearer " + self.auth.access_token})
-            if response.status_code == 200:
+            if is_success(response.status_code):
                 client = sseclient.SSEClient(response)
                 logging.info("consuming events...")
 
                 for notify_listener in self.notify_listeners:
                     notify_listener.on_connected()
-                connect_time = datetime.now()
 
                 for event in client.events():
                     if event.event == "NOTIFY":
@@ -339,6 +356,7 @@ class HomeConnect:
                             notify_listener.on_event_event(event)
                     else:
                         logging.info("unknown event type " + str(event.event))
+
                     if datetime.now() > (connect_time + timedelta(seconds=max_lifetime_sec)):
                         logging.info("closing event stream. Max lifetime " + str(max_lifetime_sec) + " sec reached (periodic reconnect)")
                         return
@@ -350,27 +368,31 @@ class HomeConnect:
                 notify_listener.on_disconnected()
             logging.info("event stream closed")
             if client is not None:
-                    client.close()
+                client.close()
 
     def devices(self) -> List[Device]:
         uri = HomeConnect.API_URI + "/homeappliances"
         logging.info("requesting " + uri)
-        response = requests.get(uri, headers={"Authorization": "Bearer " + self.auth.access_token})
-        response.raise_for_status()
-        data = response.json()
-        devices = list()
-        for homeappliances in data['data']['homeappliances']:
-            device = create_device(HomeConnect.API_URI + "/homeappliances/" + homeappliances['haId'],
-                                   self.auth,
-                                   homeappliances['name'],
-                                   homeappliances['type'],
-                                   homeappliances['haId'],
-                                   homeappliances['brand'],
-                                   homeappliances['vib'],
-                                   homeappliances['enumber'])
-            self.notify_listeners.append(device)
-            devices.append(device)
-        return devices
+        response = requests.get(uri, headers={"Authorization": "Bearer " + self.auth.access_token}, timeout=5000)
+        if is_success(response.status_code):
+            data = response.json()
+            devices = list()
+            for homeappliances in data['data']['homeappliances']:
+                device = create_device(HomeConnect.API_URI + "/homeappliances/" + homeappliances['haId'],
+                                       self.auth,
+                                       homeappliances['name'],
+                                       homeappliances['type'],
+                                       homeappliances['haId'],
+                                       homeappliances['brand'],
+                                       homeappliances['vib'],
+                                       homeappliances['enumber'])
+                self.notify_listeners.append(device)
+                devices.append(device)
+            return devices
+        else:
+            logging.warning("error occurred by calling GET " + uri)
+            logging.warning("got " + str(response.status_code) + " " + response.text)
+            raise Exception("error occurred by calling GET " + uri + " Got " + str(response))
 
     def dishwashers(self) -> List[Dishwasher]:
         return [device for device in self.devices() if isinstance(device, Dishwasher)]
